#ifndef CAM_PREPROCESS_CLASS_H
#define CAM_PREPROCESS_CLASS_H

#include <cstdint>
#include <iostream>
#include <vector>
#include "stdio.h"
#include "acl.h"
#include "acl_base.h"
#include "acl_rt.h"
#include "hi_media_common.h"
#include "hi_dvpp_venc.h"
#include "hi_dvpp_vpc.h"
#include "hi_mpi_sys.h"
#include "camera.h"
#include <ara/log/logging.h>

using namespace mdc;
using namespace mdc::camera;

class CamPreprocess
{
public:
    CamPreprocess(uint8_t id = 0);
    ~CamPreprocess();
    void setLoggerName(std::string loggerName)
    {
        this->loggerName = loggerName;
    };
    void init(uint8_t channel_count, unsigned int inputImageWidth = 1920, unsigned int inputImageHeight = 1080, unsigned int cropWidth = 960, unsigned int cropHeight = 540, unsigned int cropTopOffset = 60, unsigned int cropLeftOffset = 90, bool initEncoder = false, unsigned int encoderGopNumber = 30);
    int32_t runPipeline(uint8_t cameraId, std::shared_ptr<hi_venc_stream> *out_enc);
    int32_t runEncoderPipeline(uint8_t cameraId, std::shared_ptr<hi_venc_stream> *out_enc);
    int32_t runPerceptionPipeline(uint8_t cameraId);
    uint32_t allocateMemory(uint32_t id);

    std::vector<hi_vpc_pic_info> inputPicInfo, outputPicInfo;

private:
    uint8_t id;
    std::string loggerName = "PRE";
    std::vector<hi_vpc_crop_resize_region> cropResizeRegion;
    std::vector<hi_map_param> mappingParameters;
    std::vector<hi_remap_lut> remappingTableInformation;
    std::vector<hi_warp_transform_param> warpTransformParam;
    hi_vpc_workspace_param workSpaceParam[4];

    unsigned int inputImageWidth, inputImageHeight, cropWidth, cropHeight, cropTopOffset, cropLeftOffset, encoderGopNumber;

    aclrtRunMode runMode;
    uint32_t frame_ref = 0;
    std::vector<int32_t> epollFd;
    std::vector<hi_dvpp_epoll_event> event;
    std::vector<int32_t> fd;
    mdc::camera::CameraData cam_data;

    std::vector<hi_vpc_chn> channelIDs;
    std::vector<hi_vpc_chn_attr> channelAttr;

    aclrtContext g_context;

    uint8_t initACL();
    uint8_t initMPI(uint8_t channel_count);

    int32_t createVPCChannel();
    int32_t createEncoderChannel(int width, int height, int bufferSize, uint8_t channel_count);

    void setDefaultInputPicInfo(uint8_t channel_count);
    void setDefaultOutputPicInfo(uint8_t channel_count);
    void setDefaultCropReizeRegion(uint8_t channel_count);
    void setDefaultRemapParam(uint8_t channel_count);

    uint8_t allocateByPicInfo(uint8_t id, std::vector<hi_vpc_pic_info> &picInfo);
    uint8_t allocateByCropResizeRegion(uint8_t id, std::vector<hi_vpc_crop_resize_region> &cropResizeRegion);

    int32_t getResizedCroppedImage(uint8_t cameraId);
    int32_t getConvertedImage(uint8_t cameraId);
    int32_t getUndistortedImage(uint8_t cameraId);
    uint8_t h265Encoder(std::shared_ptr<hi_venc_stream> *out_enc, hi_venc_chn _chn);

protected:
    ara::log::Logger &logger{ara::log::CreateLogger(this->loggerName.c_str(), "SoftwareMotion Camera PreProcess Module", ara::log::LogLevel::kInfo)};
};

#endif /* CAM_PREPROCESS_CLASS_H */